extern crate fps;
extern crate time;
use fps::n::packet::*;
use fps::n::*;
const TEST_N:usize = 10000000;

fn prepare_packet1<'a>(input: &'a mut Buffer) -> Field<'a> {
    let mut buffer = &mut input[0..54];
    move_field(
	    &[
	    0xCD, 0xCD, 0xCD, 0xCD, 0xCD, 0xCD, 0xBC, 0xBC, 
	    0xBC, 0xBC, 0xBC, 0xBC, 0x08, 0x00, 0x45, 0x00, 
	    0x00, 0x28, 0x00, 0x00, 0x40, 0x00, 0x40, 0x06, 
	    0x6F, 0x1D, 0xC0, 0xA8, 0x00, 0x07, 0x0A, 0x00, 
	    0x01, 0x04, 0x56, 0xCE, 0x30, 0x39, 0xFF, 0xFF, 
	    0xF2, 0x6E, 0x00, 0x00, 0x00, 0x00, 0x50, 0x02, 
	    0xC8, 0x00, 0xA2, 0xB8, 0x00, 0x00],
	    buffer);
    buffer
}

extern {
    fn c_extern_test(test_no: i32) -> i32;
}

fn rust_c_extern_test(test_no: i32) -> bool {
    unsafe {
        c_extern_test(test_no) == 0
    }
}

fn bench_ethernet_swap_rust() {
    let mut temp_buffer = ZERO_BUFFER;
	
	let buffer = prepare_packet1(&mut temp_buffer);
	
	let mut packet = Packet::new(buffer);
	packet = phy::extract(packet);
	packet = ethernet::extract(packet);
	for _ in 0..TEST_N {
		packet = ethernet::swap_address(packet);
	}
}

fn bench_dummy_move() {
    let mut temp_buffer = ZERO_BUFFER;
	
	let buffer = prepare_packet1(&mut temp_buffer);
	
	let mut packet = Packet::new(buffer);
	packet = phy::extract(packet);
	packet = ethernet::extract(packet);
	for _ in 0..TEST_N {
		packet = packet::dummy_move(packet);
	}
}

fn bench_dummy_strip() {
    let mut temp_buffer = ZERO_BUFFER;
	
	let buffer = prepare_packet1(&mut temp_buffer);
	
	let mut packet = Packet::new(buffer);
	packet = phy::extract(packet);
	packet = ethernet::extract(packet);
	for _ in 0..TEST_N {
		packet = packet.strip(0);
	}
}

fn bench_dummy_next() {
    let mut temp_buffer = ZERO_BUFFER;
	
	let buffer = prepare_packet1(&mut temp_buffer);
	
	let mut packet = Packet::new(buffer);
	packet = phy::extract(packet);
	packet = ethernet::extract(packet);
	for _ in 0..TEST_N {
		let (in_packet,_) = packet.next_packet(0);
		packet = in_packet;
	}
}

use time::PreciseTime;

pub fn main() {
    let start = PreciseTime::now();
    bench_ethernet_swap_rust();
    let end = PreciseTime::now();

    println!("Rust: {} us for swaping address {} times.", 
        start.to(end).num_microseconds().expect("Umm"), TEST_N);
    
    let start = PreciseTime::now();
    rust_c_extern_test(4);
    let end = PreciseTime::now();

    println!("C with Rust: {} us for swaping address {} times.", 
        start.to(end).num_microseconds().expect("Umm"), TEST_N);
    
    let start = PreciseTime::now();
    rust_c_extern_test(5);
    let end = PreciseTime::now();

    println!("C native: {} us for swaping address {} times.", 
        start.to(end).num_microseconds().expect("Umm"), TEST_N);
    
    let start = PreciseTime::now();
    bench_dummy_move();
    let end = PreciseTime::now();

    println!("Rust dummy move: {} us for {} times.", 
        start.to(end).num_microseconds().expect("Umm"), TEST_N);
    
    let start = PreciseTime::now();
    bench_dummy_strip();
    let end = PreciseTime::now();

    println!("Rust dummy strip: {} us for {} times.", 
        start.to(end).num_microseconds().expect("Umm"), TEST_N);
    
    let start = PreciseTime::now();
    bench_dummy_next();
    let end = PreciseTime::now();

    println!("Rust dummy next: {} us for {} times.", 
        start.to(end).num_microseconds().expect("Umm"), TEST_N);
}