use n::packet::*;

#[inline(always)]
fn extract_destination(packet : Packet) -> Packet {
	let (mut packet,destination) = packet.next_packet(6);
	packet.context.set_field("ethernet::destination", destination);
	packet
}

#[inline(always)]
fn extract_source(packet : Packet) -> Packet {
	let (mut packet,source) = packet.next_packet(6);
	packet.context.set_field("ethernet::source", source);
	packet
}

#[inline(always)]
fn extract_protocol(packet : Packet) -> Packet {
	let (mut packet,protocol) = packet.next_packet(2);
	packet.context.set_field("ethernet::protocol", protocol);
	packet
}

#[inline(always)]
pub fn extract(packet : Packet) -> Packet {
	let next_box = packet;
	let next_box = extract_destination(next_box);
	let next_box = extract_source(next_box);
	let next_box = extract_protocol(next_box);
	next_box
}

#[inline(always)]
pub fn extract_protocol_u16(mut packet : Packet) -> Packet {
	let protocol_u16;
	{
		let protocol_field = packet.context.get_field("ethernet::protocol");
		protocol_u16 = protocol_field.read_u16(0);
	}
	packet.context.set_u16("ethernet::protocol_u16", protocol_u16);
	packet
}

#[inline(always)]
pub fn swap_address(mut packet : Packet) -> Packet {
    let mut temp_addr = &mut [0u8; 6];
	{
		let mut old_dst = packet.context.take_field("ethernet::destination");
		let mut old_src = packet.context.take_field("ethernet::source");
		move_field(old_dst, temp_addr);
		move_field(old_src, old_dst);
		move_field(temp_addr, old_src);
		packet.context.set_field("ethernet::destination", old_dst);
		packet.context.set_field("ethernet::source", old_src);
	}
	packet
}