use n::packet::*;
use n::l4;
use std::cmp::min;

#[inline(always)]
pub fn extract_source(packet : Packet) -> Packet {
    let mut packet = packet;
    packet.context.set_alias("l4::source", "udp::source");
	l4::extract_source(packet)
}

#[inline(always)]
pub fn extract_source_u16(packet : Packet) -> Packet {
    let mut packet = packet;
    packet.context.set_alias("l4::source_u16", "udp::source_u16");
	l4::extract_source_u16(packet)
}

#[inline(always)]
pub fn extract_destination(packet : Packet) -> Packet {
    let mut packet = packet;
    packet.context.set_alias("l4::destination", "udp::destination");
	l4::extract_destination(packet)
}

#[inline(always)]
pub fn extract_destination_u16(packet : Packet) -> Packet {
    let mut packet = packet;
    packet.context.set_alias("l4::destination_u16", "udp::destination_u16");
	l4::extract_destination_u16(packet)
}

#[inline(always)]
pub fn extract_length(packet : Packet) -> Packet {
    let (mut packet,field) = packet.next_packet(2);
	packet.context.set_field("udp::length", field);
	packet
}

#[inline(always)]
pub fn extract_length_u16(packet : Packet) -> Packet {
    let mut packet = packet;
	let len;
	{
		let field = packet.context.get_field("udp::length");
		len = field.read_u16(0);
	}
	packet.context.set_u16("udp::length_u16", len);
	packet
}

#[inline(always)]
pub fn extract_checksum(packet : Packet) -> Packet {
    let (mut packet,field) = packet.next_packet(2);
	packet.context.set_field("udp::checksum", field);
	packet
}

#[inline(always)]
pub fn extract_checksum_u16(packet : Packet) -> Packet {
    let mut packet = packet;
	let checksum;
	{
		let field = packet.context.get_field("udp::checksum");
		checksum = field.read_u16(0);
	}
	packet.context.set_u16("udp::checksum_u16", checksum);
	packet
}

#[inline(always)]
pub fn verify_header_checksum(packet : Packet) -> Packet {
    let mut packet = packet;
    let payload_length = min(
        packet.len(),
        packet.context.get_u32("ip::payload_length_u32") as usize
    );
    let mut sum = packet.ones_complement_sum_u16(payload_length) as u32;
    sum = sum + packet.context.get_u16("ip::pseudo_csum_u16") as u32;
    sum = (sum >> 16) + (sum & 0x0000FFFF);
    let result = sum as u16;
    packet.context.set_u16("udp::current_csum_u16", result);
    packet.context.set_bool("udp::correct_csum_bool", 
        (result == 0xFFFF) || (result == 0x0000));
    packet
}

#[inline(always)]
pub fn extract(packet : Packet) -> Packet {
    let packet = verify_header_checksum(packet);
    let packet = extract_source(packet);
	let packet = extract_destination(packet);
	let packet = extract_length(packet);
	let packet = extract_checksum(packet);
	
	packet
}